@@ -4,12 +4,10 @@
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Matthias Clasen <mclasen@redhat.com>
  *         Clemens N. Buss <cebuzz@gmail.com>
@@ -33,14 +33,17 @@
 
 
 /**
- * GEmblemedIcon:
+ * SECTION:gemblemedicon
+ * @short_description: Icon with emblems
+ * @include: gio/gio.h
+ * @see_also: #GIcon, #GLoadableIcon, #GThemedIcon, #GEmblem
  *
- * `GEmblemedIcon` is an implementation of [iface@Gio.Icon] that supports
+ * #GEmblemedIcon is an implementation of #GIcon that supports
  * adding an emblem to an icon. Adding multiple emblems to an
- * icon is ensured via [method@Gio.EmblemedIcon.add_emblem].
+ * icon is ensured via g_emblemed_icon_add_emblem(). 
  *
- * Note that `GEmblemedIcon` allows no control over the position
- * of the emblems. See also [class@Gio.Emblem] for more information.
+ * Note that #GEmblemedIcon allows no control over the position
+ * of the emblems. See also #GEmblem for more information.
  **/
 
 enum {
@@ -58,7 +61,6 @@ static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 static void g_emblemed_icon_icon_iface_init (GIconIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GEmblemedIcon, g_emblemed_icon, G_TYPE_OBJECT,
-                         G_ADD_PRIVATE (GEmblemedIcon)
                          G_IMPLEMENT_INTERFACE (G_TYPE_ICON,
                          g_emblemed_icon_icon_iface_init))
 
@@ -70,8 +72,9 @@ g_emblemed_icon_finalize (GObject *object)
 
   emblemed = G_EMBLEMED_ICON (object);
 
-  g_clear_object (&emblemed->priv->icon);
-  g_list_free_full (emblemed->priv->emblems, g_object_unref);
+  g_object_unref (emblemed->priv->icon);
+  g_list_foreach (emblemed->priv->emblems, (GFunc) g_object_unref, NULL);
+  g_list_free (emblemed->priv->emblems);
 
   (*G_OBJECT_CLASS (g_emblemed_icon_parent_class)->finalize) (object);
 }
@@ -123,35 +126,34 @@ g_emblemed_icon_class_init (GEmblemedIconClass *klass)
   gobject_class->set_property = g_emblemed_icon_set_property;
   gobject_class->get_property = g_emblemed_icon_get_property;
 
-  /**
-   * GEmblemedIcon:gicon:
-   *
-   * The [iface@Gio.Icon] to attach emblems to.
-   *
-   * Since: 2.18
-   */
   properties[PROP_GICON] =
-    g_param_spec_object ("gicon", NULL, NULL,
+    g_param_spec_object ("gicon",
+                         P_("The base GIcon"),
+                         P_("The GIcon to attach emblems to"),
                          G_TYPE_ICON,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class, NUM_PROPERTIES, properties);
+
+  g_type_class_add_private (klass, sizeof (GEmblemedIconPrivate));
 }
 
 static void
 g_emblemed_icon_init (GEmblemedIcon *emblemed)
 {
-  emblemed->priv = g_emblemed_icon_get_instance_private (emblemed);
+  emblemed->priv =
+    G_TYPE_INSTANCE_GET_PRIVATE (emblemed, G_TYPE_EMBLEMED_ICON,
+                                 GEmblemedIconPrivate);
 }
 
 /**
  * g_emblemed_icon_new:
  * @icon: a #GIcon
- * @emblem: (nullable): a #GEmblem, or %NULL
+ * @emblem: (allow-none): a #GEmblem, or %NULL
  *
  * Creates a new emblemed icon for @icon with the emblem @emblem.
  *
- * Returns: (transfer full) (type GEmblemedIcon): a new #GIcon
+ * Returns: (transfer full): a new #GIcon
  *
  * Since: 2.18
  **/
@@ -200,7 +202,7 @@ g_emblemed_icon_get_icon (GEmblemedIcon *emblemed)
  * Gets the list of emblems for the @icon.
  *
  * Returns: (element-type Gio.Emblem) (transfer none): a #GList of
- *     #GEmblems that is owned by @emblemed
+ *          #GEmblem <!-- -->s that is owned by @emblemed
  *
  * Since: 2.18
  **/
@@ -254,7 +256,7 @@ g_emblem_comp (GEmblem *a,
  * @emblemed: a #GEmblemedIcon
  * @emblem: a #GEmblem
  *
- * Adds @emblem to the #GList of #GEmblems.
+ * Adds @emblem to the #GList of #GEmblem <!-- -->s.
  *
  * Since: 2.18
  **/
@@ -363,7 +365,7 @@ g_emblemed_icon_from_tokens (gchar  **tokens,
       g_set_error (error,
                    G_IO_ERROR,
                    G_IO_ERROR_INVALID_ARGUMENT,
-                   _("Can’t handle version %d of GEmblemedIcon encoding"),
+                   _("Can't handle version %d of GEmblemedIcon encoding"),
                    version);
       goto fail;
     }
@@ -412,54 +414,6 @@ g_emblemed_icon_from_tokens (gchar  **tokens,
   return NULL;
 }
 
-static GVariant *
-g_emblemed_icon_serialize (GIcon *icon)
-{
-  GEmblemedIcon *emblemed_icon = G_EMBLEMED_ICON (icon);
-  GVariantBuilder builder;
-  GVariant *icon_data;
-  GList *node;
-
-  icon_data = g_icon_serialize (emblemed_icon->priv->icon);
-  if (!icon_data)
-    return NULL;
-
-  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("(va(va{sv}))"));
-
-  g_variant_builder_add (&builder, "v", icon_data);
-  g_variant_unref (icon_data);
-
-  g_variant_builder_open (&builder, G_VARIANT_TYPE ("a(va{sv})"));
-  for (node = emblemed_icon->priv->emblems; node != NULL; node = node->next)
-    {
-      icon_data = g_icon_serialize (node->data);
-      if (icon_data)
-        {
-          /* We know how emblems serialize, so do a tweak here to
-           * reduce some of the variant wrapping and redundant storage
-           * of 'emblem' over and again...
-           */
-          if (g_variant_is_of_type (icon_data, G_VARIANT_TYPE ("(sv)")))
-            {
-              const gchar *name;
-              GVariant *content;
-
-              g_variant_get (icon_data, "(&sv)", &name, &content);
-
-              if (g_str_equal (name, "emblem") && g_variant_is_of_type (content, G_VARIANT_TYPE ("(va{sv})")))
-                g_variant_builder_add (&builder, "@(va{sv})", content);
-
-              g_variant_unref (content);
-            }
-
-          g_variant_unref (icon_data);
-        }
-    }
-  g_variant_builder_close (&builder);
-
-  return g_variant_new ("(sv)", "emblemed", g_variant_builder_end (&builder));
-}
-
 static void
 g_emblemed_icon_icon_iface_init (GIconIface *iface)
 {
@@ -467,5 +421,4 @@ g_emblemed_icon_icon_iface_init (GIconIface *iface)
   iface->equal = g_emblemed_icon_equal;
   iface->to_tokens = g_emblemed_icon_to_tokens;
   iface->from_tokens = g_emblemed_icon_from_tokens;
-  iface->serialize = g_emblemed_icon_serialize;
 }
