@@ -2,12 +2,10 @@
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  */
@@ -25,25 +25,28 @@
 #include <glib.h>
 #include <gfileinputstream.h>
 #include <gseekable.h>
+#include "gsimpleasyncresult.h"
 #include "gcancellable.h"
 #include "gasyncresult.h"
-#include "gtask.h"
 #include "gioerror.h"
 #include "glibintl.h"
 
 
 /**
- * GFileInputStream:
+ * SECTION:gfileinputstream
+ * @short_description: File input streaming operations
+ * @include: gio/gio.h
+ * @see_also: #GInputStream, #GDataInputStream, #GSeekable
  *
- * `GFileInputStream` provides input streams that take their
+ * GFileInputStream provides input streams that take their
  * content from a file.
  *
- * `GFileInputStream` implements [iface@Gio.Seekable], which allows the input
+ * GFileInputStream implements #GSeekable, which allows the input 
  * stream to jump to arbitrary positions in the file, provided the 
  * filesystem of the file allows it. To find the position of a file
- * input stream, use [method@Gio.Seekable.tell]. To find out if a file input
- * stream supports seeking, use [vfunc@Gio.Seekable.can_seek].
- * To position a file input stream, use [vfunc@Gio.Seekable.seek].
+ * input stream, use g_seekable_tell(). To find out if a file input
+ * stream supports seeking, use g_seekable_stream_can_seek().
+ * To position a file input stream, use g_seekable_seek().
  **/
 
 static void       g_file_input_stream_seekable_iface_init    (GSeekableIface       *iface);
@@ -70,18 +73,19 @@ static GFileInfo *g_file_input_stream_real_query_info_finish (GFileInputStream
 							      GError              **error);
 
 
+G_DEFINE_TYPE_WITH_CODE (GFileInputStream, g_file_input_stream, G_TYPE_INPUT_STREAM,
+			 G_IMPLEMENT_INTERFACE (G_TYPE_SEEKABLE,
+						g_file_input_stream_seekable_iface_init))
+
 struct _GFileInputStreamPrivate {
   GAsyncReadyCallback outstanding_callback;
 };
 
-G_DEFINE_TYPE_WITH_CODE (GFileInputStream, g_file_input_stream, G_TYPE_INPUT_STREAM,
-                         G_ADD_PRIVATE (GFileInputStream)
-			 G_IMPLEMENT_INTERFACE (G_TYPE_SEEKABLE,
-						g_file_input_stream_seekable_iface_init))
-
 static void
 g_file_input_stream_class_init (GFileInputStreamClass *klass)
 {
+  g_type_class_add_private (klass, sizeof (GFileInputStreamPrivate));
+
   klass->query_info_async = g_file_input_stream_real_query_info_async;
   klass->query_info_finish = g_file_input_stream_real_query_info_finish;
 }
@@ -99,15 +103,17 @@ g_file_input_stream_seekable_iface_init (GSeekableIface *iface)
 static void
 g_file_input_stream_init (GFileInputStream *stream)
 {
-  stream->priv = g_file_input_stream_get_instance_private (stream);
+  stream->priv = G_TYPE_INSTANCE_GET_PRIVATE (stream,
+					      G_TYPE_FILE_INPUT_STREAM,
+					      GFileInputStreamPrivate);
 }
 
 /**
  * g_file_input_stream_query_info:
  * @stream: a #GFileInputStream.
  * @attributes: a file attribute query string.
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore. 
- * @error: a #GError location to store the error occurring, or %NULL to 
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore. 
+ * @error: a #GError location to store the error occuring, or %NULL to 
  * ignore.
  *
  * Queries a file input stream the given @attributes. This function blocks 
@@ -145,7 +151,7 @@ g_file_input_stream_query_info (GFileInputStream  *stream,
     info = class->query_info (stream, attributes, cancellable, error);
   else
     g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                         _("Stream doesn’t support query_info"));
+                         _("Stream doesn't support query_info"));
 
   if (cancellable)
     g_cancellable_pop_current (cancellable);
@@ -172,11 +178,11 @@ async_ready_callback_wrapper (GObject      *source_object,
  * g_file_input_stream_query_info_async:
  * @stream: a #GFileInputStream.
  * @attributes: a file attribute query string.
- * @io_priority: the [I/O priority](iface.AsyncResult.html#io-priority) of the request
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore. 
- * @callback: (scope async): a #GAsyncReadyCallback
- *   to call when the request is satisfied
- * @user_data: the data to pass to callback function
+ * @io_priority: the <link linkend="io-priority">I/O priority</link> 
+ *     of the request.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore. 
+ * @callback: (scope async): callback to call when the request is satisfied
+ * @user_data: (closure): the data to pass to callback function
  * 
  * Queries the stream information asynchronously.
  * When the operation is finished @callback will be called. 
@@ -209,9 +215,10 @@ g_file_input_stream_query_info_async (GFileInputStream    *stream,
   
   if (!g_input_stream_set_pending (input_stream, &error))
     {
-      g_task_report_error (stream, callback, user_data,
-                           g_file_input_stream_query_info_async,
-                           error);
+      g_simple_async_report_take_gerror_in_idle (G_OBJECT (stream),
+					    callback,
+					    user_data,
+					    error);
       return;
     }
 
@@ -220,14 +227,14 @@ g_file_input_stream_query_info_async (GFileInputStream    *stream,
   stream->priv->outstanding_callback = callback;
   g_object_ref (stream);
   klass->query_info_async (stream, attributes, io_priority, cancellable,
-                           async_ready_callback_wrapper, user_data);
+			      async_ready_callback_wrapper, user_data);
 }
 
 /**
  * g_file_input_stream_query_info_finish:
  * @stream: a #GFileInputStream.
  * @result: a #GAsyncResult.
- * @error: a #GError location to store the error occurring, 
+ * @error: a #GError location to store the error occuring, 
  *     or %NULL to ignore.
  * 
  * Finishes an asynchronous info query operation.
@@ -239,15 +246,18 @@ g_file_input_stream_query_info_finish (GFileInputStream  *stream,
                                        GAsyncResult      *result,
                                        GError           **error)
 {
+  GSimpleAsyncResult *simple;
   GFileInputStreamClass *class;
 
   g_return_val_if_fail (G_IS_FILE_INPUT_STREAM (stream), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
 
-  if (g_async_result_legacy_propagate_error (result, error))
-    return NULL;
-  else if (g_async_result_is_tagged (result, g_file_input_stream_query_info_async))
-    return g_task_propagate_pointer (G_TASK (result), error);
+  if (G_IS_SIMPLE_ASYNC_RESULT (result))
+    {
+      simple = G_SIMPLE_ASYNC_RESULT (result);
+      if (g_simple_async_result_propagate_error (simple, error))
+	return NULL;
+    }
 
   class = G_FILE_INPUT_STREAM_GET_CLASS (stream);
   return class->query_info_finish (stream, result, error);
@@ -374,29 +384,45 @@ g_file_input_stream_seekable_truncate (GSeekable     *seekable,
  *   Default implementation of async ops    *
  ********************************************/
 
+typedef struct {
+  char *attributes;
+  GFileInfo *info;
+} QueryInfoAsyncData;
+
 static void
-query_info_async_thread (GTask        *task,
-                         gpointer      source_object,
-                         gpointer      task_data,
-                         GCancellable *cancellable)
+query_info_data_free (QueryInfoAsyncData *data)
+{
+  if (data->info)
+    g_object_unref (data->info);
+  g_free (data->attributes);
+  g_free (data);
+}
+
+static void
+query_info_async_thread (GSimpleAsyncResult *res,
+		         GObject            *object,
+		         GCancellable       *cancellable)
 {
-  GFileInputStream *stream = source_object;
-  const char *attributes = task_data;
   GFileInputStreamClass *class;
   GError *error = NULL;
-  GFileInfo *info = NULL;
+  QueryInfoAsyncData *data;
+  GFileInfo *info;
+  
+  data = g_simple_async_result_get_op_res_gpointer (res);
 
-  class = G_FILE_INPUT_STREAM_GET_CLASS (stream);
+  info = NULL;
+  
+  class = G_FILE_INPUT_STREAM_GET_CLASS (object);
   if (class->query_info)
-    info = class->query_info (stream, attributes, cancellable, &error);
+    info = class->query_info (G_FILE_INPUT_STREAM (object), data->attributes, cancellable, &error);
   else
     g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                         _("Stream doesn’t support query_info"));
+                         _("Stream doesn't support query_info"));
 
   if (info == NULL)
-    g_task_return_error (task, error);
+    g_simple_async_result_take_error (res, error);
   else
-    g_task_return_pointer (task, info, g_object_unref);
+    data->info = info;
 }
 
 static void
@@ -407,15 +433,17 @@ g_file_input_stream_real_query_info_async (GFileInputStream    *stream,
                                            GAsyncReadyCallback  callback,
                                            gpointer             user_data)
 {
-  GTask *task;
+  GSimpleAsyncResult *res;
+  QueryInfoAsyncData *data;
 
-  task = g_task_new (stream, cancellable, callback, user_data);
-  g_task_set_source_tag (task, g_file_input_stream_real_query_info_async);
-  g_task_set_task_data (task, g_strdup (attributes), g_free);
-  g_task_set_priority (task, io_priority);
+  data = g_new0 (QueryInfoAsyncData, 1);
+  data->attributes = g_strdup (attributes);
   
-  g_task_run_in_thread (task, query_info_async_thread);
-  g_object_unref (task);
+  res = g_simple_async_result_new (G_OBJECT (stream), callback, user_data, g_file_input_stream_real_query_info_async);
+  g_simple_async_result_set_op_res_gpointer (res, data, (GDestroyNotify)query_info_data_free);
+  
+  g_simple_async_result_run_in_thread (res, query_info_async_thread, io_priority, cancellable);
+  g_object_unref (res);
 }
 
 static GFileInfo *
@@ -423,7 +451,14 @@ g_file_input_stream_real_query_info_finish (GFileInputStream  *stream,
                                             GAsyncResult      *res,
                                             GError           **error)
 {
-  g_return_val_if_fail (g_task_is_valid (res, stream), NULL);
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
+  QueryInfoAsyncData *data;
 
-  return g_task_propagate_pointer (G_TASK (res), error);
+  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == g_file_input_stream_real_query_info_async);
+
+  data = g_simple_async_result_get_op_res_gpointer (simple);
+  if (data->info)
+    return g_object_ref (data->info);
+  
+  return NULL;
 }
