@@ -24,7 +24,6 @@
 #include "ginitable.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:ginitable
@@ -44,7 +43,8 @@
  * in various ways. For C applications you generally just call
  * g_initable_new() directly, or indirectly via a foo_thing_new() wrapper.
  * This will call g_initable_init() under the cover, returning %NULL and
- * setting a %GError on failure.
+ * setting a #GError on failure (at which point the instance is
+ * unreferenced).
  *
  * For bindings in languages where the native constructor supports
  * exceptions the binding could check for objects implemention %GInitable
@@ -52,35 +52,12 @@
  * an exception on failure.
  */
 
-GType
-g_initable_get_type (void)
+typedef GInitableIface GInitableInterface;
+G_DEFINE_INTERFACE (GInitable, g_initable, G_TYPE_OBJECT)
+
+static void
+g_initable_default_init (GInitableInterface *iface)
 {
-  static volatile gsize g_define_type_id__volatile = 0;
-
-  if (g_once_init_enter (&g_define_type_id__volatile))
-    {
-      const GTypeInfo initable_info =
-      {
-	sizeof (GInitableIface), /* class_size */
-	NULL,           /* base_init */
-	NULL,		/* base_finalize */
-	NULL,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	0,
-	0,              /* n_preallocs */
-	NULL
-      };
-      GType g_define_type_id =
-	g_type_register_static (G_TYPE_INTERFACE, I_("GInitable"),
-				&initable_info, 0);
-
-      g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_OBJECT);
-
-      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
-    }
-
-  return g_define_type_id__volatile;
 }
 
 /**
@@ -144,7 +121,7 @@ g_initable_init (GInitable     *initable,
  * similar to g_object_new() but also initializes the object
  * and returns %NULL, setting an error on failure.
  *
- * Return value: a newly allocated #GObject, or %NULL on error
+ * Return value: (transfer full): a newly allocated #GObject, or %NULL on error
  *
  * Since: 2.22
  */
@@ -180,7 +157,7 @@ g_initable_new (GType          object_type,
  * similar to g_object_newv() but also initializes the object
  * and returns %NULL, setting an error on failure.
  *
- * Return value: a newly allocated #GObject, or %NULL on error
+ * Return value: (transfer full): a newly allocated #GObject, or %NULL on error
  *
  * Since: 2.22
  */
@@ -220,7 +197,7 @@ g_initable_newv (GType          object_type,
  * similar to g_object_new_valist() but also initializes the object
  * and returns %NULL, setting an error on failure.
  *
- * Return value: a newly allocated #GObject, or %NULL on error
+ * Return value: (transfer full): a newly allocated #GObject, or %NULL on error
  *
  * Since: 2.22
  */
@@ -247,6 +224,3 @@ g_initable_new_valist (GType          object_type,
 
   return obj;
 }
-
-#define __G_INITABLE_C__
-#include "gioaliasdef.c"
