@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the licence, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors: Vincent Untz <vuntz@gnome.org>
  *          Ryan Lortie <desrt@desrt.ca>
@@ -21,21 +23,14 @@
 
 #include "config.h"
 
-#include <glib.h>
-#include <glibintl.h>
-
 #include <stdio.h>
 #include <string.h>
 
 #include "gfile.h"
 #include "gfileinfo.h"
-#include "gfileenumerator.h"
 #include "gfilemonitor.h"
 #include "gsimplepermission.h"
-#include "gsettingsbackendinternal.h"
-#include "gstrfuncsprivate.h"
-#include "giomodule-priv.h"
-#include "gportalsupport.h"
+#include "gsettingsbackend.h"
 
 
 #define G_TYPE_KEYFILE_SETTINGS_BACKEND      (g_keyfile_settings_backend_get_type ())
@@ -48,13 +43,6 @@
 
 typedef GSettingsBackendClass GKeyfileSettingsBackendClass;
 
-typedef enum {
-  PROP_FILENAME = 1,
-  PROP_ROOT_PATH,
-  PROP_ROOT_GROUP,
-  PROP_DEFAULTS_DIR
-} GKeyfileSettingsBackendProperty;
-
 typedef struct
 {
   GSettingsBackend   parent_instance;
@@ -62,9 +50,6 @@ typedef struct
   GKeyFile          *keyfile;
   GPermission       *permission;
   gboolean           writable;
-  char              *defaults_dir;
-  GKeyFile          *system_keyfile;
-  GHashTable        *system_locks; /* Used as a set, owning the strings it contains */
 
   gchar             *prefix;
   gint               prefix_len;
@@ -78,18 +63,10 @@ typedef struct
   GFileMonitor      *dir_monitor;
 } GKeyfileSettingsBackend;
 
-#ifdef G_OS_WIN32
-#define EXTENSION_PRIORITY 10
-#else
-#define EXTENSION_PRIORITY (glib_should_use_portal () && !glib_has_dconf_access_in_sandbox () ? 110 : 10)
-#endif
-
-G_DEFINE_TYPE_WITH_CODE (GKeyfileSettingsBackend,
-                         g_keyfile_settings_backend,
-                         G_TYPE_SETTINGS_BACKEND,
-                         _g_io_modules_ensure_extension_points_registered ();
-                         g_io_extension_point_implement (G_SETTINGS_BACKEND_EXTENSION_POINT_NAME,
-                                                         g_define_type_id, "keyfile", EXTENSION_PRIORITY))
+static GType g_keyfile_settings_backend_get_type (void);
+G_DEFINE_TYPE (GKeyfileSettingsBackend,
+               g_keyfile_settings_backend,
+               G_TYPE_SETTINGS_BACKEND)
 
 static void
 compute_checksum (guint8        *digest,
@@ -114,8 +91,7 @@ g_keyfile_settings_backend_keyfile_write (GKeyfileSettingsBackend *kfsb)
 
   contents = g_key_file_to_data (kfsb->keyfile, &length, NULL);
   g_file_replace_contents (kfsb->file, contents, length, NULL, FALSE,
-                           G_FILE_CREATE_REPLACE_DESTINATION |
-                           G_FILE_CREATE_PRIVATE,
+                           G_FILE_CREATE_REPLACE_DESTINATION,
                            NULL, NULL, NULL);
 
   compute_checksum (kfsb->digest, contents, length);
@@ -146,8 +122,8 @@ convert_path (GKeyfileSettingsBackend  *kfsb,
               gchar                   **group,
               gchar                   **basename)
 {
-  gsize key_len = strlen (key);
-  const gchar *last_slash;
+  gint key_len = strlen (key);
+  gint i;
 
   if (key_len < kfsb->prefix_len ||
       memcmp (key, kfsb->prefix, kfsb->prefix_len) != 0)
@@ -156,53 +132,43 @@ convert_path (GKeyfileSettingsBackend  *kfsb,
   key_len -= kfsb->prefix_len;
   key += kfsb->prefix_len;
 
-  last_slash = strrchr (key, '/');
-
-  /* Disallow empty group names or key names */
-  if (key_len == 0 ||
-      (last_slash != NULL &&
-       (*(last_slash + 1) == '\0' ||
-        last_slash == key)))
-    return FALSE;
+  for (i = key_len; i >= 0; i--)
+    if (key[i] == '/')
+      break;
 
   if (kfsb->root_group)
     {
       /* if a root_group was specified, make sure the user hasn't given
        * a path that ghosts that group name
        */
-      if (last_slash != NULL && (last_slash - key) == kfsb->root_group_len && memcmp (key, kfsb->root_group, last_slash - key) == 0)
+      if (i == kfsb->root_group_len && memcmp (key, kfsb->root_group, i) == 0)
         return FALSE;
     }
   else
     {
       /* if no root_group was given, ensure that the user gave a path */
-      if (last_slash == NULL)
+      if (i == -1)
         return FALSE;
     }
 
   if (group)
     {
-      if (last_slash != NULL)
+      if (i >= 0)
         {
-          *group = g_memdup2 (key, (last_slash - key) + 1);
-          (*group)[(last_slash - key)] = '\0';
+          *group = g_memdup (key, i + 1);
+          (*group)[i] = '\0';
         }
       else
         *group = g_strdup (kfsb->root_group);
     }
 
   if (basename)
-    {
-      if (last_slash != NULL)
-        *basename = g_memdup2 (last_slash + 1, key_len - (last_slash - key));
-      else
-        *basename = g_strdup (key);
-    }
+    *basename = g_memdup (key + i + 1, key_len - i);
 
   return TRUE;
 }
 
-static gboolean
+gboolean
 path_is_valid (GKeyfileSettingsBackend *kfsb,
                const gchar             *path)
 {
@@ -220,51 +186,17 @@ get_from_keyfile (GKeyfileSettingsBackend *kfsb,
   if (convert_path (kfsb, key, &group, &name))
     {
       gchar *str;
-      gchar *sysstr;
 
       g_assert (*name);
 
-      sysstr = g_key_file_get_value (kfsb->system_keyfile, group, name, NULL);
       str = g_key_file_get_value (kfsb->keyfile, group, name, NULL);
-      if (sysstr &&
-          (g_hash_table_contains (kfsb->system_locks, key) ||
-           str == NULL))
-        {
-          g_free (str);
-          str = g_steal_pointer (&sysstr);
-        }
 
       if (str)
         {
           return_value = g_variant_parse (type, str, NULL, NULL, NULL);
-
-          /* As a special case, support values of type %G_VARIANT_TYPE_STRING
-           * not being quoted, since users keep forgetting to do it and then
-           * getting confused. */
-          if (return_value == NULL &&
-              g_variant_type_equal (type, G_VARIANT_TYPE_STRING) &&
-              str[0] != '\"')
-            {
-              GString *s = g_string_sized_new (strlen (str) + 2);
-              char *p = str;
-
-              g_string_append_c (s, '\"');
-              while (*p)
-                {
-                  if (*p == '\"')
-                    g_string_append_c (s, '\\');
-                  g_string_append_c (s, *p);
-                  p++;
-                }
-              g_string_append_c (s, '\"');
-              return_value = g_variant_parse (type, s->str, NULL, NULL, NULL);
-              g_string_free (s, TRUE);
-            }
           g_free (str);
         }
 
-      g_free (sysstr);
-
       g_free (group);
       g_free (name);
     }
@@ -279,9 +211,6 @@ set_to_keyfile (GKeyfileSettingsBackend *kfsb,
 {
   gchar *group, *name;
 
-  if (g_hash_table_contains (kfsb->system_locks, key))
-    return FALSE;
-
   if (convert_path (kfsb, key, &group, &name))
     {
       if (value)
@@ -345,7 +274,7 @@ g_keyfile_settings_backend_write_one (gpointer key,
                                       gpointer user_data)
 {
   WriteManyData *data = user_data;
-  gboolean success G_GNUC_UNUSED  /* when compiling with G_DISABLE_ASSERT */;
+  gboolean success;
 
   success = set_to_keyfile (data->kfsb, key, value);
   g_assert (success);
@@ -360,8 +289,7 @@ g_keyfile_settings_backend_check_one (gpointer key,
 {
   WriteManyData *data = user_data;
 
-  return data->failed = g_hash_table_contains (data->kfsb->system_locks, key) ||
-                        !path_is_valid (data->kfsb, key);
+  return data->failed = !path_is_valid (data->kfsb, key);
 }
 
 static gboolean
@@ -429,9 +357,7 @@ g_keyfile_settings_backend_get_writable (GSettingsBackend *backend,
 {
   GKeyfileSettingsBackend *kfsb = G_KEYFILE_SETTINGS_BACKEND (backend);
 
-  return kfsb->writable &&
-         !g_hash_table_contains (kfsb->system_locks, name) &&
-         path_is_valid (kfsb, name);
+  return kfsb->writable && path_is_valid (kfsb, name);
 }
 
 static GPermission *
@@ -468,7 +394,6 @@ keyfile_to_tree (GKeyfileSettingsBackend *kfsb,
         continue;
 
       keys = g_key_file_get_keys (keyfile, groups[i], NULL, NULL);
-      g_assert (keys != NULL);
 
       for (j = 0; keys[j]; j++)
         {
@@ -577,9 +502,6 @@ g_keyfile_settings_backend_finalize (GObject *object)
 
   g_key_file_free (kfsb->keyfile);
   g_object_unref (kfsb->permission);
-  g_key_file_unref (kfsb->system_keyfile);
-  g_hash_table_unref (kfsb->system_locks);
-  g_free (kfsb->defaults_dir);
 
   g_file_monitor_cancel (kfsb->file_monitor);
   g_object_unref (kfsb->file_monitor);
@@ -601,6 +523,24 @@ g_keyfile_settings_backend_init (GKeyfileSettingsBackend *kfsb)
 {
 }
 
+static void
+g_keyfile_settings_backend_class_init (GKeyfileSettingsBackendClass *class)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+  object_class->finalize = g_keyfile_settings_backend_finalize;
+
+  class->read = g_keyfile_settings_backend_read;
+  class->write = g_keyfile_settings_backend_write;
+  class->write_tree = g_keyfile_settings_backend_write_tree;
+  class->reset = g_keyfile_settings_backend_reset;
+  class->get_writable = g_keyfile_settings_backend_get_writable;
+  class->get_permission = g_keyfile_settings_backend_get_permission;
+  /* No need to implement subscribed/unsubscribe: the only point would be to
+   * stop monitoring the file when there's no GSettings anymore, which is no
+   * big win. */
+}
+
 static void
 file_changed (GFileMonitor      *monitor,
               GFile             *file,
@@ -610,9 +550,7 @@ file_changed (GFileMonitor      *monitor,
 {
   GKeyfileSettingsBackend *kfsb = user_data;
 
-  /* Ignore file deletions, let the GKeyFile content remain in tact. */
-  if (event_type != G_FILE_MONITOR_EVENT_DELETED)
-    g_keyfile_settings_backend_keyfile_reload (kfsb);
+  g_keyfile_settings_backend_keyfile_reload (kfsb);
 }
 
 static void
@@ -627,289 +565,13 @@ dir_changed (GFileMonitor       *monitor,
   g_keyfile_settings_backend_keyfile_writable (kfsb);
 }
 
-static void
-load_system_settings (GKeyfileSettingsBackend *kfsb)
-{
-  GError *error = NULL;
-  const char *dir = "/etc/glib-2.0/settings";
-  char *path;
-  char *contents;
-
-  kfsb->system_keyfile = g_key_file_new ();
-  kfsb->system_locks = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-
-  if (kfsb->defaults_dir)
-    dir = kfsb->defaults_dir;
-
-  path = g_build_filename (dir, "defaults", NULL);
-
-  /* The defaults are in the same keyfile format that we use for the settings.
-   * It can be produced from a dconf database using: dconf dump
-   */
-  if (!g_key_file_load_from_file (kfsb->system_keyfile, path, G_KEY_FILE_NONE, &error))
-    {
-      if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-        g_warning ("Failed to read %s: %s", path, error->message);
-      g_clear_error (&error);
-    }
-  else
-    g_debug ("Loading default settings from %s", path);
-
-  g_free (path);
-
-  path = g_build_filename (dir, "locks", NULL);
-
-  /* The locks file is a text file containing a list paths to lock, one per line.
-   * It can be produced from a dconf database using: dconf list-locks
-   */
-  if (!g_file_get_contents (path, &contents, NULL, &error))
-    {
-      if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-        g_warning ("Failed to read %s: %s", path, error->message);
-      g_clear_error (&error);
-    }
-  else
-    {
-      char **lines;
-      gsize i;
-
-      g_debug ("Loading locks from %s", path);
-
-      lines = g_strsplit (contents, "\n", 0);
-      for (i = 0; lines[i]; i++)
-        {
-          char *line = lines[i];
-          if (line[0] == '#' || line[0] == '\0')
-            {
-              g_free (line);
-              continue;
-            }
-
-          g_debug ("Locking key %s", line);
-          g_hash_table_add (kfsb->system_locks, g_steal_pointer (&line));
-        }
-
-      g_free (lines);
-    }
-  g_free (contents);
-
-  g_free (path);
-}
-
-static void
-g_keyfile_settings_backend_constructed (GObject *object)
-{
-  GKeyfileSettingsBackend *kfsb = G_KEYFILE_SETTINGS_BACKEND (object);
-
-  if (kfsb->file == NULL)
-    {
-      char *filename = g_build_filename (g_get_user_config_dir (),
-                                         "glib-2.0", "settings", "keyfile",
-                                         NULL);
-      kfsb->file = g_file_new_for_path (filename);
-      g_free (filename);
-    }
-
-  if (kfsb->prefix == NULL)
-    {
-      kfsb->prefix = g_strdup ("/");
-      kfsb->prefix_len = 1;
-    }
-  
-  kfsb->keyfile = g_key_file_new ();
-  kfsb->permission = g_simple_permission_new (TRUE);
-
-  kfsb->dir = g_file_get_parent (kfsb->file);
-  g_mkdir_with_parents (g_file_peek_path (kfsb->dir), 0700);
-
-  kfsb->file_monitor = g_file_monitor (kfsb->file, G_FILE_MONITOR_NONE, NULL, NULL);
-  kfsb->dir_monitor = g_file_monitor (kfsb->dir, G_FILE_MONITOR_NONE, NULL, NULL);
-
-  compute_checksum (kfsb->digest, NULL, 0);
-
-  g_signal_connect (kfsb->file_monitor, "changed",
-                    G_CALLBACK (file_changed), kfsb);
-  g_signal_connect (kfsb->dir_monitor, "changed",
-                    G_CALLBACK (dir_changed), kfsb);
-
-  g_keyfile_settings_backend_keyfile_writable (kfsb);
-  g_keyfile_settings_backend_keyfile_reload (kfsb);
-
-  load_system_settings (kfsb);
-}
-
-static void
-g_keyfile_settings_backend_set_property (GObject      *object,
-                                         guint         prop_id,
-                                         const GValue *value,
-                                         GParamSpec   *pspec)
-{
-  GKeyfileSettingsBackend *kfsb = G_KEYFILE_SETTINGS_BACKEND (object);
-
-  switch ((GKeyfileSettingsBackendProperty)prop_id)
-    {
-    case PROP_FILENAME:
-      /* Construct only. */
-      g_assert (kfsb->file == NULL);
-      if (g_value_get_string (value))
-        kfsb->file = g_file_new_for_path (g_value_get_string (value));
-      break;
-
-    case PROP_ROOT_PATH:
-      /* Construct only. */
-      g_assert (kfsb->prefix == NULL);
-      kfsb->prefix = g_value_dup_string (value);
-      if (kfsb->prefix)
-        kfsb->prefix_len = strlen (kfsb->prefix);
-      break;
-
-    case PROP_ROOT_GROUP:
-      /* Construct only. */
-      g_assert (kfsb->root_group == NULL);
-      kfsb->root_group = g_value_dup_string (value);
-      if (kfsb->root_group)
-        kfsb->root_group_len = strlen (kfsb->root_group);
-      break;
-
-    case PROP_DEFAULTS_DIR:
-      /* Construct only. */
-      g_assert (kfsb->defaults_dir == NULL);
-      kfsb->defaults_dir = g_value_dup_string (value);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-static void
-g_keyfile_settings_backend_get_property (GObject    *object,
-                                         guint       prop_id,
-                                         GValue     *value,
-                                         GParamSpec *pspec)
-{
-  GKeyfileSettingsBackend *kfsb = G_KEYFILE_SETTINGS_BACKEND (object);
-
-  switch ((GKeyfileSettingsBackendProperty)prop_id)
-    {
-    case PROP_FILENAME:
-      g_value_set_string (value, g_file_peek_path (kfsb->file));
-      break;
-
-    case PROP_ROOT_PATH:
-      g_value_set_string (value, kfsb->prefix);
-      break;
-
-    case PROP_ROOT_GROUP:
-      g_value_set_string (value, kfsb->root_group);
-      break;
-
-    case PROP_DEFAULTS_DIR:
-      g_value_set_string (value, kfsb->defaults_dir);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-static void
-g_keyfile_settings_backend_class_init (GKeyfileSettingsBackendClass *class)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (class);
-
-  object_class->finalize = g_keyfile_settings_backend_finalize;
-  object_class->constructed = g_keyfile_settings_backend_constructed;
-  object_class->get_property = g_keyfile_settings_backend_get_property;
-  object_class->set_property = g_keyfile_settings_backend_set_property;
-
-  class->read = g_keyfile_settings_backend_read;
-  class->write = g_keyfile_settings_backend_write;
-  class->write_tree = g_keyfile_settings_backend_write_tree;
-  class->reset = g_keyfile_settings_backend_reset;
-  class->get_writable = g_keyfile_settings_backend_get_writable;
-  class->get_permission = g_keyfile_settings_backend_get_permission;
-  /* No need to implement subscribed/unsubscribe: the only point would be to
-   * stop monitoring the file when there's no GSettings anymore, which is no
-   * big win.
-   */
-
-  /**
-   * GKeyfileSettingsBackend:filename:
-   *
-   * The location where the settings are stored on disk.
-   *
-   * Defaults to `$XDG_CONFIG_HOME/glib-2.0/settings/keyfile`.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_FILENAME,
-                                   g_param_spec_string ("filename",
-                                                        P_("Filename"),
-                                                        P_("The filename"),
-                                                        NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GKeyfileSettingsBackend:root-path:
-   *
-   * All settings read to or written from the backend must fall under the
-   * path given in @root_path (which must start and end with a slash and
-   * not contain two consecutive slashes).  @root_path may be "/".
-   * 
-   * Defaults to "/".
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_ROOT_PATH,
-                                   g_param_spec_string ("root-path",
-                                                        P_("Root path"),
-                                                        P_("The root path"),
-                                                        NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GKeyfileSettingsBackend:root-group:
-   *
-   * If @root_group is non-%NULL then it specifies the name of the keyfile
-   * group used for keys that are written directly below the root path.
-   *
-   * Defaults to NULL.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_ROOT_GROUP,
-                                   g_param_spec_string ("root-group",
-                                                        P_("Root group"),
-                                                        P_("The root group"),
-                                                        NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GKeyfileSettingsBackend:default-dir:
-   *
-   * The directory where the system defaults and locks are located.
-   *
-   * Defaults to `/etc/glib-2.0/settings`.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_DEFAULTS_DIR,
-                                   g_param_spec_string ("defaults-dir",
-                                                        P_("Default dir"),
-                                                        P_("Defaults dir"),
-                                                        NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
-}
-
 /**
  * g_keyfile_settings_backend_new:
  * @filename: the filename of the keyfile
  * @root_path: the path under which all settings keys appear
- * @root_group: (nullable): the group name corresponding to
+ * @root_group: (allow-none): the group name corresponding to
  *              @root_path, or %NULL
+ * @returns: (transfer full): a keyfile-backed #GSettingsBackend
  *
  * Creates a keyfile-backed #GSettingsBackend.
  *
@@ -955,28 +617,49 @@ g_keyfile_settings_backend_class_init (GKeyfileSettingsBackendClass *class)
  * syntax of the key file format.  For example, if you have '[' or ']'
  * characters in your path names or '=' in your key names you may be in
  * trouble.
- *
- * The backend reads default values from a keyfile called `defaults` in
- * the directory specified by the #GKeyfileSettingsBackend:defaults-dir property,
- * and a list of locked keys from a text file with the name `locks` in
- * the same location.
- *
- * Returns: (transfer full): a keyfile-backed #GSettingsBackend
  **/
 GSettingsBackend *
 g_keyfile_settings_backend_new (const gchar *filename,
                                 const gchar *root_path,
                                 const gchar *root_group)
 {
+  GKeyfileSettingsBackend *kfsb;
+
   g_return_val_if_fail (filename != NULL, NULL);
   g_return_val_if_fail (root_path != NULL, NULL);
   g_return_val_if_fail (g_str_has_prefix (root_path, "/"), NULL);
   g_return_val_if_fail (g_str_has_suffix (root_path, "/"), NULL);
   g_return_val_if_fail (strstr (root_path, "//") == NULL, NULL);
 
-  return G_SETTINGS_BACKEND (g_object_new (G_TYPE_KEYFILE_SETTINGS_BACKEND,
-                                           "filename", filename,
-                                           "root-path", root_path,
-                                           "root-group", root_group,
-                                           NULL));
+  kfsb = g_object_new (G_TYPE_KEYFILE_SETTINGS_BACKEND, NULL);
+  kfsb->keyfile = g_key_file_new ();
+  kfsb->permission = g_simple_permission_new (TRUE);
+
+  kfsb->file = g_file_new_for_path (filename);
+  kfsb->dir = g_file_get_parent (kfsb->file);
+  g_file_make_directory_with_parents (kfsb->dir, NULL, NULL);
+
+  kfsb->file_monitor = g_file_monitor_file (kfsb->file, 0, NULL, NULL);
+  kfsb->dir_monitor = g_file_monitor_file (kfsb->dir, 0, NULL, NULL);
+
+  kfsb->prefix_len = strlen (root_path);
+  kfsb->prefix = g_strdup (root_path);
+
+  if (root_group)
+    {
+      kfsb->root_group_len = strlen (root_group);
+      kfsb->root_group = g_strdup (root_group);
+    }
+
+  compute_checksum (kfsb->digest, NULL, 0);
+
+  g_signal_connect (kfsb->file_monitor, "changed",
+                    G_CALLBACK (file_changed), kfsb);
+  g_signal_connect (kfsb->dir_monitor, "changed",
+                    G_CALLBACK (dir_changed), kfsb);
+
+  g_keyfile_settings_backend_keyfile_writable (kfsb);
+  g_keyfile_settings_backend_keyfile_reload (kfsb);
+
+  return G_SETTINGS_BACKEND (kfsb);
 }
