@@ -2,12 +2,10 @@
  *
  * Copyright © 2009 Codethink Limited
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation; either version 2 of the licence or (at
+ * your option) any later version.
  *
  * See the included COPYING file for more information.
  *
@@ -15,28 +13,31 @@
  */
 
 /**
- * GSocketControlMessage:
+ * SECTION:gsocketcontrolmessage
+ * @title: GSocketControlMessage
+ * @short_description: A GSocket control message
+ * @see_also: #GSocket.
  *
- * A `GSocketControlMessage` is a special-purpose utility message that
- * can be sent to or received from a [class@Gio.Socket]. These types of
- * messages are often called ‘ancillary data’.
+ * A #GSocketControlMessage is a special-purpose utility message that
+ * can be sent to or received from a #GSocket. These types of
+ * messages are often called "ancillary data".
  *
  * The message can represent some sort of special instruction to or
  * information from the socket or can represent a special kind of
- * transfer to the peer (for example, sending a file descriptor over
+ * transfer to the peer (for example, sending a file description over
  * a UNIX socket).
  *
- * These messages are sent with [method@Gio.Socket.send_message] and received
- * with [method@Gio.Socket.receive_message].
+ * These messages are sent with g_socket_send_message() and received
+ * with g_socket_receive_message().
  *
- * To extend the set of control messages that can be sent, subclass this
- * class and override the `get_size`, `get_level`, `get_type` and `serialize`
+ * To extend the set of control message that can be sent, subclass this
+ * class and override the get_size, get_level, get_type and serialize
  * methods.
  *
  * To extend the set of control messages that can be received, subclass
- * this class and implement the `deserialize` method. Also, make sure your
- * class is registered with the [type@GObject.Type] type system before calling
- * [method@Gio.Socket.receive_message] to read such a message.
+ * this class and implement the deserialize method. Also, make sure your
+ * class is registered with the GType typesystem before calling
+ * g_socket_receive_message() to read such a message.
  *
  * Since: 2.22
  */
@@ -52,7 +53,9 @@
 #endif
 
 
-G_DEFINE_ABSTRACT_TYPE (GSocketControlMessage, g_socket_control_message, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE (GSocketControlMessage,
+                        g_socket_control_message,
+                        G_TYPE_OBJECT);
 
 /**
  * g_socket_control_message_get_size:
@@ -114,7 +117,7 @@ g_socket_control_message_get_msg_type (GSocketControlMessage *message)
 /**
  * g_socket_control_message_serialize:
  * @message: a #GSocketControlMessage
- * @data: (not nullable): A buffer to write data to
+ * @data: A buffer to write data to
  *
  * Converts the data in the message to bytes placed in the
  * message.
@@ -160,7 +163,7 @@ g_socket_control_message_class_init (GSocketControlMessageClass *class)
  * If there is no implementation for this kind of control message, %NULL
  * will be returned.
  *
- * Returns: (nullable) (transfer full): the deserialized message or %NULL
+ * Returns: (transfer full): the deserialized message or %NULL
  *
  * Since: 2.22
  */
@@ -173,12 +176,15 @@ g_socket_control_message_deserialize (int      level,
   GSocketControlMessage *message;
   GType *message_types;
   guint n_message_types;
-  guint i;
+  int i;
+#ifndef G_OS_WIN32
+  volatile GType a_type;
+#endif
 
   /* Ensure we know about the built in types */
 #ifndef G_OS_WIN32
-  g_type_ensure (G_TYPE_UNIX_CREDENTIALS_MESSAGE);
-  g_type_ensure (G_TYPE_UNIX_FD_MESSAGE);
+  a_type = g_unix_credentials_message_get_type ();
+  a_type = g_unix_fd_message_get_type ();
 #endif
 
   message_types = g_type_children (G_TYPE_SOCKET_CONTROL_MESSAGE, &n_message_types);
@@ -198,15 +204,8 @@ g_socket_control_message_deserialize (int      level,
 
   g_free (message_types);
 
-  /* It's not a bug if we can't deserialize the control message - for
-   * example, the control message may be be discarded if it is deemed
-   * empty, see e.g.
-   *
-   *  https://gitlab.gnome.org/GNOME/glib/commit/ec91ed00f14c70cca9749347b8ebc19d72d9885b
-   *
-   * Therefore, it's not appropriate to print a warning about not
-   * being able to deserialize the message.
-   */
+  if (message == NULL)
+    g_warning ("unknown control message type %d:%d", level, type);
 
   return message;
 }
